
InputConfigWindow = {}
InputConfigWindow.MetaTable = {}

function InputConfigWindow.Create(pName,pInputScheme,pCallbackFunc,pOnClose,pCloseInfo,pOnChange)
  local new = {}
  if getmetatable(pInputScheme) ~= InputScheme.MetaTable then
    error("Argument 2 of InputConfigWindow.Create is not a InputScheme");
  end
  if type(pCallbackFunc) ~= "string" then
    error("Argument 3 of InputConfigWindow.Create is not a string");
  end
  if type(pOnClose) ~= "string" then
    error("Argument 4 of InputConfigWindow.Create is not a string");
  end
  if pOnChange ~= nil then
    if type(pOnChange) ~= "function" then
      error("Argument 5 of InputConfigWindow.Create ist not a function");
    end
  end
  new._InputScheme = pInputScheme
  new._Name = pName
  new._KeyList = {}
  new._GuiButtons = {}
  new._KeyCount = 0
  new._CallbackFunc = pCallbackFunc
  new._OnClose = pOnClose
  if pOnChange ~= nil then
    new._OnChange = pOnChange
  end
  new._NextY = 50
  new._GuiWindow = gui.CreateWindowL(-1,10,10,300,100,false,0,"input_config_title")
  new._GuiCloseButton = gui.CreateButtonL(new._GuiWindow,190,40,100,20,0,1,"input_config_close")
  gui.ButtonSetOnPress(new._GuiCloseButton,pOnClose)
  gui.CreateTextL(new._GuiWindow,10,5,gui.GetFont("Arial10"),"input_config_text0")
  gui.CreateTextL(new._GuiWindow,10,25,gui.GetFont("Arial10"),"input_config_text1")
  new._GuiInfoWindow = gui.CreateWindowL(-1,100,100,200,90,false,0,"input_config_info_title")
  new._GuiInfoText = gui.CreateTextL(new._GuiInfoWindow,10,5,gui.GetFont("Arial10"),"input_config_info_press_key")
  new._GuiInfoKey = gui.CreateText(new._GuiInfoWindow,10,25,gui.GetFont("Arial10"),"key")
  new._GuiInfoCloseButton = gui.CreateButtonL(new._GuiInfoWindow,95,50,100,20,0,1,"input_config_info_close")
  gui.ButtonSetOnPress(new._GuiInfoCloseButton,pCloseInfo)
  setmetatable(new,InputConfigWindow.MetaTable)
  gui.ElementSetActive(new._GuiInfoWindow,false)
  new:Hide()
  return new
end

function InputConfigWindow.Dummy(key)
end

function InputConfigWindow:Add(pName,pDefaultKey,pFunction,pRepeat)
  if self._InputScheme:GetBinaryEventByKey(pDefaultKey) ~= nil then
    error("The Key "..pDefaultKey.." is already in use")
  end
  if self._InputScheme:GetBinaryEventByName(pName) ~= nil then
    error("A binary event with the name '"..pName.."' does already exist")
  end
  self._KeyCount = self._KeyCount + 1
  self._KeyList[self._KeyCount] = pName
  gui.CreateTextL(self._GuiWindow,10,self._NextY+3,gui.GetFont("Arial10"),pName)
  self._GuiButtons[self._KeyCount] = gui.CreateButtonL(self._GuiWindow,170,self._NextY,120,20,0,1,"key_"..pDefaultKey)
  gui.ButtonSetOnPress(self._GuiButtons[self._KeyCount],self._CallbackFunc,self._KeyCount)
  self._NextY = self._NextY + 25
  self._InputScheme:SetBinaryEvent(pName,pDefaultKey,pFunction,pRepeat)
  gui.ElementSetRect(self._GuiWindow,10,10,300,self._NextY+40)
  gui.ElementSetPosition(self._GuiCloseButton,190,self._NextY)
end

function InputConfigWindow:Show()
  gui.ElementSetActive(self._GuiWindow,true)
  self.CenterGuiWindow(self._GuiWindow)
end

function InputConfigWindow:Hide()
  if self._Current ~= nil then
    return false
  else
    gui.ElementSetActive(self._GuiWindow,false)
    return true
  end
end

function InputConfigWindow:CloseInfo()
  gui.ElementSetActive(self._GuiInfoWindow,false)
  self._Current = nil
end

function InputConfigWindow.CenterGuiWindow(id)
  local width = gui.ElementGetWidth(id)
  local height = gui.ElementGetHeight(id)
  local newx = (Globals["WindowWidth"] - width) / 2
  local newy = (Globals["WindowHeight"]  - height) / 2
  gui.ElementSetRect(id,newx,newy,width,height)
end

function InputConfigWindow:Callback(id)
  if self._Current ~= nil then
    return
  end
  self._Current = id
  OnBinaryEventOnReciever = self
  OnBinaryEventOffReciever = InputConfigWindow.Dummy
  gui.ElementSetActive(self._GuiInfoWindow,true)
  gui.ElementToForeground(self._GuiInfoWindow)
  self.CenterGuiWindow(self._GuiInfoWindow)
  gui.ElementSetActive(self._GuiInfoKey,false)
  gui.ElementSetActive(self._GuiInfoCloseButton,false)
  gui.TextSetCaptionL(self._GuiInfoText,"input_config_info_press_key")
end

function InputConfigWindow:KeyIn(key)
  local old = self._InputScheme:GetBinaryEventByKey(key)
  if old == self._KeyList[self._Current] then
    self:CloseInfo()
    OnBinaryEventOnReciever = nil
    OnBinaryEventOffReciever = nil
  end
  if old ~= nil then
    OnBinaryEventOnReciever = nil
    OnBinaryEventOffReciever = nil
    gui.ElementSetActive(self._GuiInfoKey,true)
    gui.ElementSetActive(self._GuiInfoCloseButton,true)
    gui.TextSetCaptionL(self._GuiInfoKey,old)
    gui.TextSetCaptionL(self._GuiInfoText,"input_config_info_key_already_in_use")
    return
  end
  local name = self._KeyList[self._Current]
  local _,func,rep = self._InputScheme:GetBinaryEventByName(name)
  self._InputScheme:DeleteBinaryEvent(name)
  self._InputScheme:SetBinaryEvent(name,key,func,rep)
  if self._OnChange ~= nil then
    self._OnChange(name,key,func,rep)
  end
  gui.ButtonSetCaptionL(self._GuiButtons[self._Current],"key_"..key)
  OnBinaryEventOnReciever = nil
  OnBinaryEventOffReciever = nil
  self:CloseInfo()
end

function InputConfigWindow:Save(pFilename)
  local t = {}
  for i,v in ipairs(self._KeyList) do
    local j = #t+1
    t[j] = {}
    t[j].key = self._InputScheme:GetBinaryEventByName(v)
    t[j].name = v
  end
  table.save(t,pFilename)
end

function InputConfigWindow:Load(pFilename)
  local t = table.load(pFilename)
  if t == nil then
    return false
  else
    for i,v in ipairs(t) do
      local _,func,rep = self._InputScheme:GetBinaryEventByName(v.name)
      self._InputScheme:DeleteBinaryEvent(v.name)
      self._InputScheme:SetBinaryEvent(v.name,v.key,func,rep)
      if self._OnChange ~= nil then
        self._OnChange(v.name,v.key,func,rep)
      end
      gui.ButtonSetCaptionL(self._GuiButtons[i],"key_"..v.key)
    end
    return true
  end
end

InputConfigWindow.MetaTable["__index"] = InputConfigWindow
InputConfigWindow.MetaTable["__call"] = InputConfigWindow.KeyIn
