dofile("lua/inputscheme.lua")
dofile("lua/tetrisinput.lua")
dofile("lua/InputConfigWindow.lua")

TetrisColors = {}
TetrisStones = {}

TetrisField = {}
CurrentStone = {}
CurrentStoneId = -1
CurrentStonePos = vec2(0,28)
CurrentStoneSize = vec2(-1,-1)
NextStoneSize = vec2(-1,-1)

TetrisStone = -1
NextStone = -1
CurrentStone = -1
GameOver = false

ModelNoTextureXml = -1
ModelNoTextureGlowXml = -1
ModelNoTextureGlow = -1

DownTimeDiff = 550
DeleteStones = -1
Glow = false
LastGlow = 0
Level = 1
Points = 0
LastDown = 0

InfoWindow = -1
NameWindow = -1
NameInput = -1
NameButton = -1
GameOverWindow = -1
GameOverButton = -1
LevelInfo = -1
PunkteInfo = -1

SpielzeitH = 0
SpielzeitM = 0
SpielzeitS = 0

Melodie = -1
SoundOn = true
PlayList = {}
LastSong = 1

HighscoreNames = {}
HighscorePoints = {}

function InputCallback(id)
  theInputConfigWindow:Callback(id)
end

function InputClose()
  theInputConfigWindow:Hide()
  gui.ElementSetActive(GameOverWindow,true)
end

function InputCloseInfo()
  theInputConfigWindow:CloseInfo()
end

function ShowControls()
  gui.ElementSetActive(GameOverWindow,false)
  theInputConfigWindow:Show()
end

function InputChange(pName,pKey,pFunc,pRepeat)
  if pName == "input_quit" then
    PauseScheme:DeleteBinaryEvent("input_quit")
    PauseScheme:SetBinaryEvent(pName,pKey,pFunc,pRepeat)
    GameOverScheme:DeleteBinaryEvent("input_quit")
    GameOverScheme:SetBinaryEvent(pName,pKey,pFunc,pRepeat)
  elseif pName == "input_pause" then
    PauseScheme:DeleteBinaryEvent("input_pause")
    PauseScheme:SetBinaryEvent("input_pause",pKey,pFunc,pRepeat)
  end
end

function InitTetris()
  SetupTetrisInput()

  GameScheme = InputScheme.Create()
  theInputConfigWindow = InputConfigWindow.Create("Input1",GameScheme,"InputCallback","InputClose","InputCloseInfo",InputChange)
  theInputConfigWindow:Add("input_move_left",203,MoveStoneLeft,130)
  theInputConfigWindow:Add("input_move_right",205,MoveStoneRight,130)
  theInputConfigWindow:Add("input_move_down",208,StoneMoveDown,50)
  theInputConfigWindow:Add("input_pause",197,TetrisPause)
  theInputConfigWindow:Add("input_quit",1,TetrisQuit)
  theInputConfigWindow:Add("input_complete_down",29,StoneMoveCompleteDown)
  theInputConfigWindow:Add("input_rotate_left",44,RotateCurrentStoneLeft)
  theInputConfigWindow:Add("input_rotate_right",45,RotateCurrentStoneRight)
  if not theInputConfigWindow:Load("input.dat") then
    io.write("loading input.dat failed\n")
  end

  GameScheme:Use()

  TetrisColors[0] = vec4(1,0,0,1) -- Rot
  TetrisColors[1] = vec4(0,1,0,1) -- Grn
  TetrisColors[2] = vec4(0,0,1,1) -- Blau
  TetrisColors[3] = vec4(1,1,0,1) -- Gelb
  TetrisColors[4] = vec4(0,1,1,1) -- Cyan
  TetrisColors[5] = vec4(1,1,1,1) -- Wei
  TetrisColors[6] = vec4(1,0.8,1,0) -- Orange

  for i=0,6 do
    TetrisStones[i] = {}
    for x=0,3 do
      TetrisStones[i][x] = {}
      for y=0,3 do
        TetrisStones[i][x][y] = -1
      end
    end
  end

  TetrisStones[0].Size = vec2(4,2)
  TetrisStones[0][0][1] = 0
  TetrisStones[0][1][1] = 0
  TetrisStones[0][2][1] = 0
  TetrisStones[0][3][1] = 0

  TetrisStones[1].Size = vec2(3,3)
  TetrisStones[1][0][1] = 1
  TetrisStones[1][0][2] = 1
  TetrisStones[1][1][1] = 1
  TetrisStones[1][2][1] = 1

  TetrisStones[2].Size = vec2(3,3)
  TetrisStones[2][0][1] = 2
  TetrisStones[2][1][1] = 2
  TetrisStones[2][2][1] = 2
  TetrisStones[2][2][2] = 2

  TetrisStones[3].Size = vec2(2,2)
  TetrisStones[3][0][0] = 3
  TetrisStones[3][0][1] = 3
  TetrisStones[3][1][0] = 3
  TetrisStones[3][1][1] = 3

  TetrisStones[4].Size = vec2(3,2)
  TetrisStones[4][0][0] = 4
  TetrisStones[4][1][0] = 4
  TetrisStones[4][1][1] = 4
  TetrisStones[4][2][1] = 4

  TetrisStones[5].Size = vec2(3,3)
  TetrisStones[5][0][1] = 5
  TetrisStones[5][1][1] = 5
  TetrisStones[5][1][2] = 5
  TetrisStones[5][2][1] = 5

  TetrisStones[6].Size = vec2(3,2)
  TetrisStones[6][0][1] = 6
  TetrisStones[6][1][1] = 6
  TetrisStones[6][1][0] = 6
  TetrisStones[6][2][0] = 6

  ModelNoTextureXml = renderer.NewXmlShader("shader/modelnotexture.xml")
  renderer.XmlShaderUpload(ModelNoTextureXml)

  ModelNoTextureGlowXml = renderer.NewXmlShader("shader/modelnotextureglow.xml")
  renderer.XmlShaderUpload(ModelNoTextureGlowXml)
  ModelNoTextureGlow = renderer.XmlShaderGetShader(ModelNoTextureGlowXml)

  TetrisStone = renderer.NewModel()
  renderer.ModelLoadFile(TetrisStone,"gfx/stone.DAE")
  renderer.ModelSetShader(TetrisStone,renderer.XmlShaderGetShader(ModelNoTextureXml))

  --Setup Gui-Windows
  local Arial10 = gui.GetFont("Arial10")
  local Ypos = 20
  if Globals["WindowWidth"] < 1024 then Ypos = 50 end
  InfoWindow = gui.CreateWindowL(-1,Globals["WindowWidth"]-200,Ypos,150,100,false,0,"info_title")
  gui.CreateTextL(InfoWindow,5,5,Arial10,"info_points")
  gui.CreateTextL(InfoWindow,5,25,Arial10,"info_level")
  gui.CreateTextL(InfoWindow,5,45,Arial10,"info_playtime")
  PunkteInfo = gui.CreateText(InfoWindow,60,5,Arial10,"0")
  LevelInfo = gui.CreateText(InfoWindow,60,25,Arial10,"1")
  SpielzeitInfo = gui.CreateText(InfoWindow,60,45,Arial10,"00:00:00")

  GameOverWindow = gui.CreateWindowL(-1,(Globals["WindowWidth"]-200)/2,(Globals["WindowHeight"]-300)/2,220,300,false,0,"game_over_title")
  GameOverButton = gui.CreateButtonL(GameOverWindow,5,265,100,15,0,1,"game_over_new_game")
  GameOverControls = gui.CreateButtonL(GameOverWindow,115,265,100,15,0,1,"game_over_controls")
  gui.CreateTextL(GameOverWindow,50,5,Arial10,"game_over_highscores")
  for i=1,10 do
    HighscoreNames[i] = gui.CreateText(GameOverWindow,5,5+20*i,Arial10,"empty")
    HighscorePoints[i] = gui.CreateText(GameOverWindow,150,5+20*i,Arial10,"0")
  end
  gui.ButtonSetOnPress(GameOverButton,"NewGame")
  gui.ButtonSetOnPress(GameOverControls,"ShowControls")

  NameWindow = gui.CreateWindowL(-1,(Globals["WindowWidth"]-250)/2,(Globals["WindowHeight"]-100)/2,250,100,false,0,"name_title")
  gui.CreateTextL(NameWindow,10,5,Arial10,"name_text")
  NameInput = gui.CreateTextInputL(NameWindow,10,25,230,20,6,7,25,"name_input")
  NameButton = gui.CreateButtonL(NameWindow,140,55,100,20,0,1,"name_save")
  gui.ButtonSetOnPress(NameButton,"SaveNickname")
  gui.ElementSetActive(NameWindow,false)

  PauseWindow = gui.CreateWindowL(-1,(Globals["WindowWidth"]-200)/2,(Globals["WindowHeight"]-100)/2,200,100,false,0,"pause_title")
  gui.CreateTextL(PauseWindow,10,5,Arial10,"pause_text")
  gui.ElementSetActive(PauseWindow,false)

  for line in io.lines("sfx/playlist.txt") do
    local file = io.open("sfx/"..line,"r")
    if file ~= nil then
      PlayList[#PlayList+1] = line
      io.close(file)
    end
  end

  if #PlayList > 0 then
    Melodie = sound.NewSource(true)
    sound.LoadFile(Melodie,"sfx/"..PlayList[1])
    sound.Play(Melodie)
  else
    SoundOn = false
  end

  --Load Highscores
  Highscores = table.load("highscores.dat")
  if Highscores == nil then
    Highscores = {}
    for i=1,10 do
      Highscores[i] = {}
      Highscores[i]["name"]="empty"
      Highscores[i]["points"]=0
    end
  end

  gui.SetActive(true)
  NewGame()
  DoGameOver()
end

function DeInitTetris()
  if theInputConfigWindow ~= nil then
    theInputConfigWindow:Save("input.dat")
  end
  if Highscores ~= nil then
    assert(table.save(Highscores,"highscores.dat")==1)
  end
end

function UpdateHighscores()
  for i=1,10 do
    gui.TextSetCaption(HighscoreNames[i],Highscores[i]["name"])
    gui.TextSetCaption(HighscorePoints[i],Highscores[i]["points"])
  end
end

function SaveNickname()
  for i=1,10 do
    if Highscores[i]["points"] < Points then
      for j=10,i+1,-1 do
        Highscores[j] = Highscores[j-1]
      end
      Highscores[i] = {}
      Highscores[i]["name"] = gui.TextInputGetCaption(NameInput)
      Highscores[i]["points"] = Points
      break
    end
  end
  gui.ElementSetActive(NameWindow,false)
  UpdateHighscores()
  gui.ElementSetActive(GameOverWindow,true)
end

function NewGame()
  for x=0,9 do
    TetrisField[x] = {}
    for y=0,31 do
      TetrisField[x][y] = -1
    end
  end

  SpielzeitH = 0
  SpielzeitM = 0
  SpielzeitS = 0
  DownTimeDiff = 550
  Level = 1
  Points = 0
  LastLevel = 0
  LastDown = 0
  GameOver = false
  GameScheme:Use()
  gui.ElementSetActive(GameOverWindow,false)

  math.randomseed(os.time())
  GenNextStone()
  UseNextStone()

  UpdateSpielzeit()
  gui.TextSetCaption(PunkteInfo,Points)
  gui.TextSetCaption(LevelInfo,Level)
  if GlowingRows ~= nil then
    GlowingRows = nil
  end
  DeleteStones = -1
end

function GenNextStone()
  NextStone = {}
  local random = math.random(0,6)
  while random == CurrentStoneId do
    random = math.random(0,6)
  end
  CurrentStoneId = random
  for x=0,3 do
    NextStone[x] = {}
    for y=0,3 do
      NextStone[x][y] = TetrisStones[random][x][y]
    end
  end
  NextStoneSize = TetrisStones[random].Size()
end

function DoGameOver()
  GameOver = true
  GameOverScheme:Use()
  local found = false
  for i=1,10 do
    if Highscores[i]["points"] < Points then
      found = true
      break
    end
  end
  if found then
    gui.ElementSetActive(NameWindow,true)
  else
    gui.ElementSetActive(GameOverWindow,true)
  end
end

function UseNextStone()
  CurrentStone = NextStone
  CurrentStoneSize = NextStoneSize()
  CurrentStonePos.x = 4
  CurrentStonePos.y = 31
  local empty=true
  for i=0,3 do
    if CurrentStone[i][0] ~= -1 then empty = false end
  end
  if empty then
    CurrentStonePos.y = 30
  end
  GenNextStone()
  if CheckCollision() then
    DoGameOver()
  end
  LastDown = 0
end

function CheckLevel()
  if Level > 9.5 then return end
  while 3.98103189303^(Level+1) + 100 < Points do
    Level = Level + 1
    DownTimeDiff = DownTimeDiff - 50
    gui.TextSetCaption(LevelInfo,Level)
  end
end

function UpdateTetris(TimeDiff)
  if GameOver or Pause then return end
  LastDown = LastDown + TimeDiff
  LastLevel = LastLevel + TimeDiff / 1000
  LastGlow = LastGlow + TimeDiff
  if SoundOn then
    if not sound.IsPlaying(Melodie) then
      LastSong = LastSong +1
      if LastSong > #PlayList then
        LastSong = 1
      end
      sound.DeleteSource(Melodie)
      Melodie = sound.NewSource(true)
      sound.LoadFile(Melodie,"sfx/"..PlayList[LastSong])
      sound.Play(Melodie)
    end
  end
  while LastDown > DownTimeDiff do
    StoneMoveDown()
    LastDown = LastDown - DownTimeDiff
  end
  while LastGlow > 100 do
    Glow = not Glow
    LastGlow = LastGlow - 100
  end
  if DeleteStones > 0 then
    DeleteStones = DeleteStones - TimeDiff
    if DeleteStones <= 0 then
      RemoveAllFinishedLines()
    end
  end
  RepeatEvent.Repeat(TimeDiff)
  --Update Spielzeit
  SpielzeitS = SpielzeitS + TimeDiff / 1000
  UpdateSpielzeit()
end

function UpdateSpielzeit()
  if SpielzeitS > 60 then
    SpielzeitS = SpielzeitS - 60
    SpielzeitM = SpielzeitM + 1
  end
  if SpielzeitM > 60 then
    SpielzeitM = SpielzeitM - 60
    SpielzeitH = SpielzeitH +1
  end
  local String = ""
  if SpielzeitH < 10 then String = String.."0" end
  String = String .. math.floor(SpielzeitH)..":"
  if SpielzeitM < 10 then String = String.."0" end
  String = String .. math.floor(SpielzeitM)..":"
  if SpielzeitS < 10 then String = String.."0" end
  String = String .. math.floor(SpielzeitS)
  gui.TextSetCaption(SpielzeitInfo,String)
end

function DrawTetris(RenderGroup)
  --Draw Normal Stones
  for y=0,31 do
    local draw = true
    if GlowingRows ~= nil and Glow then
      for i,r in ipairs(GlowingRows) do
        if r == y then
          draw = false
          break
        end
      end
    end
    if draw then
      for x=0,9 do
        if TetrisField[x][y] ~= -1 then
          local RenderCall = renderer.ModelDraw(TetrisStone,StateObject2,RenderGroup)
          renderer.RenderCallOverrideMat4(RenderCall,ModelMatrix,math3d.TranslationMatrix(vec4(x,0,y,1)))
          renderer.RenderCallOverrideVec4(RenderCall,TextureColor0,TetrisColors[TetrisField[x][y]])
        end
      end
    end
  end

  --Draw Glowing Stones
  if GlowingRows ~= nil and Glow then
    for i,r in ipairs(GlowingRows) do
      for x=0,9 do
        if TetrisField[x][y] ~= -1 then
          local RenderCall = renderer.ModelDraw(TetrisStone,StateObject2,RenderGroup)
          renderer.RenderCallOverrideMat4(RenderCall,ModelMatrix,math3d.TranslationMatrix(vec4(x,0,r,1)))
          renderer.RenderCallOverrideVec4(RenderCall,TextureColor0,TetrisColors[TetrisField[x][r]])
          renderer.RenderCallSetShader(RenderCall,ModelNoTextureGlow)
        end
      end
    end
  end

  for y=0,3 do
    for x=0,3 do
      if CurrentStone[x][y] ~= -1 then
        local RenderCall = renderer.ModelDraw(TetrisStone,StateObject2,RenderGroup)
        renderer.RenderCallOverrideMat4(RenderCall,ModelMatrix,math3d.TranslationMatrix(vec4(x+CurrentStonePos.x,0,y+CurrentStonePos.y,1)))
        renderer.RenderCallOverrideVec4(RenderCall,TextureColor0,TetrisColors[CurrentStone[x][y]])
      end
    end
  end

  for y=0,3 do
    for x=0,3 do
      if NextStone[x][y] ~= -1 then
        local RenderCall = renderer.ModelDraw(TetrisStone,StateObject2,RenderGroup)
        renderer.RenderCallOverrideMat4(RenderCall,ModelMatrix,math3d.TranslationMatrix(vec4(x+12,0,y+29,1)))
        renderer.RenderCallOverrideVec4(RenderCall,TextureColor0,TetrisColors[NextStone[x][y]])
      end
    end
  end
end

function CheckStone()
  local work = true
  while work do
    for y=0,3 do
      if CurrentStone[0][y] ~= -1 then work = false end
    end
    if work then
      for y=0,3 do
        for x=0,2 do
          CurrentStone[x][y] = CurrentStone[x+1][y]
        end
        CurrentStone[3][y] = -1
      end
    end
  end
  work = true
  while work do
    for x=0,3 do
      if CurrentStone[x][0] ~= -1 then work = false end
    end
    if work then
      for x=0,3 do
        for y=0,2 do
          CurrentStone[x][y] = CurrentStone[x][y+1]
        end
        CurrentStone[x][3] = -1
      end
    end
  end
end

function RotateCurrentStoneLeft()
  local OldSize = CurrentStoneSize()
  --if CurrentStonePos.x + OldSize.y > 10.5 then return end
  CurrentStoneSize.x = OldSize.y
  CurrentStoneSize.y = OldSize.x
  local size = OldSize.y
  if OldSize.x > size then size = OldSize.x end
  size = size - 1
  local NewStone = {}
  for x=0,3 do
    NewStone[x] = {}
    for y=0,3 do
      NewStone[x][y] = -1
    end
  end
  for x=0,size do
    for y=0,size do
      NewStone[x][y] = CurrentStone[y][size-x]
    end
  end
  local temp = CurrentStone
  CurrentStone = NewStone
  --CheckStone()
  if CheckCollision() then
    CurrentStone = temp
    CurrentStoneSize = OldSize
  end
end

function RotateCurrentStoneRight()
  local OldSize = CurrentStoneSize()
  --if CurrentStonePos.y + OldSize.y > 10.5 then return end
  CurrentStoneSize.x = OldSize.y
  CurrentStoneSize.y = OldSize.x
  local size = OldSize.y
  if OldSize.x > size then size = OldSize.x end
  size = size-1
  local NewStone = {}
  for x=0,3 do
    NewStone[x] = {}
    for y=0,3 do
      NewStone[x][y] = -1
    end
  end
  for x=0,size do
    for y=0,size do
      NewStone[x][y] = CurrentStone[size-y][x]
    end
  end
  local temp = CurrentStone
  CurrentStone = NewStone
  --CheckStone()
  if CheckCollision() then
    CurrentStone = temp
    CurrentStoneSize = OldSize
  end
end

function CheckCollision()
  for y=0,3 do
    for x=0,3 do
      local tx = x + CurrentStonePos.x
      local ty = y + CurrentStonePos.y
      if tx > -0.5 and tx < 9.5 and ty > -0.5 and ty < 31.5 then
        if CurrentStone[x][y] ~= -1 and TetrisField[tx][ty] ~= -1 then return true end
      else
        if CurrentStone[x][y] ~= -1 and ty < 31.5 then
          return true
        end
      end
    end
  end
  return false
end

function MoveStoneLeft()
  CurrentStonePos.x = CurrentStonePos.x - 1
  if CheckCollision() then
    CurrentStonePos.x = CurrentStonePos.x + 1
  end
end

function MoveStoneRight()
  CurrentStonePos.x = CurrentStonePos.x + 1
  if CheckCollision() then
    CurrentStonePos.x = CurrentStonePos.x - 1
  end
end

function StoneMoveDown()
  CurrentStonePos.y = CurrentStonePos.y - 1
  if CheckCollision() then
    CurrentStonePos.y = CurrentStonePos.y + 1
    DropStone()
  end
end

function StoneMoveCompleteDown()
  local work = true
  while work do
    CurrentStonePos.y = CurrentStonePos.y - 1
    if CheckCollision() then
      CurrentStonePos.y = CurrentStonePos.y + 1
      work = false
    end
  end
  DropStone()
end

function DropStone()
  if CurrentStonePos.y + CurrentStoneSize.y > 31.5 then
    DoGameOver()
  end
  for y=0,3 do
    for x=0,3 do
      local tx = x + CurrentStonePos.x
      local ty = y + CurrentStonePos.y
      if CurrentStone[x][y] ~= -1 then
        TetrisField[tx][ty] = CurrentStone[x][y]
      end
    end
  end
  CheckGlowingRows()
  UseNextStone()
  Points = Points + 1
  gui.TextSetCaption(PunkteInfo,Points)
  CheckLevel()
end

function RemoveAllFinishedLines()
  GlowingRows = nil
  DeleteStones = -1
  local work = true
  local lines = 0
  while work do
    work = RemoveFinishedLines()
    if work then lines = lines + 1 end
  end
  Points = Points + (10^lines)
  gui.TextSetCaption(PunkteInfo,Points)
  CheckLevel()
end

function RemoveFinishedLines()
  for y=0,31 do
    local found = true
    for x=0,9 do
      if TetrisField[x][y] == -1 then
        found = false
        break
      end
    end
    if found then
      for y2=y,30 do
        for x=0,9 do
          TetrisField[x][y2] = TetrisField[x][y2+1]
        end
      end
      for x=0,9 do TetrisField[x][31] = -1 end
      return true
    end
  end
  return false
end

function CheckGlowingRows()
  local Rows = {}
  local i = 1

  for y=0,31 do
    local found = true
    for x=0,9 do
      if TetrisField[x][y] == -1 then
        found = false
        break
      end
    end
    if found then
      Rows[i] = y
      i = i + 1
    end
  end

  if i ~= 1 then
    if GlowingRows ~= nil then
      RemoveAllFinishedLines()
    end
    GlowingRows = Rows
    DeleteStones = 700
  end
end
